#ifndef SPARROWSHELL_H
#define SPARROWSHELL_H

#include <list>

#include "system.h"
#include "scene/scenetree.h"
#include "glm/glm.hpp"

class Input;
class MeshNode;
class TextNode;

namespace sf {
class Window;
}

class SparrowShell : public ContainerNode
{
private:
    class ScrollBar{
        SparrowShell* m_shell;
        glm::ivec2 m_position;
        glm::ivec2 m_dimension;
        MeshNode *m_mesh;
    public:
        ScrollBar(){}
        ScrollBar(SparrowShell* shell);
        void update();
    };

    class ShellBuffer : public ContainerNode {
    private:
        int m_zero_offset = 0;
        unsigned int m_max_size;
        float m_font_size;
    public:
        ShellBuffer(int buffer_size):m_max_size(buffer_size){}
        SceneNode* operator[](int i){return m_children[(m_zero_offset+i)%m_max_size];}
        void push(TextNode*);
        unsigned int size(){return m_children.size();}
        void toggleBuffer();

        // if this font_size is scaling down, sizes which are power of 2 render better.
        void setFontSize(float font_size){m_font_size = font_size;}
        float getFontSize(){return m_font_size;}
    };

    static const unsigned int BUFFER_MAX_LENGTH;
    static const unsigned int BUFFER_DISPLAYED_NUMBER;
    static const unsigned int SCROLLBAR_PIXEL_WIDTH;
    static const float SHELL_DEPTH;

    sf::Window* m_window;
    Input* m_input;

    glm::ivec2 m_position;
    glm::ivec2 m_dimension;
    ShellBuffer m_buffer;
    unsigned int m_index = 0;
    bool m_resizeScrollBar = false;
    bool m_indexMoved = false;
    bool m_shellEnabled = false;
    //textMesh
    MeshNode* m_background;
    ScrollBar m_scrollbar;

public:
    SparrowShell(sf::Window*, Input*);

    void update();
//    void setScene(SceneTree *scene){m_currentScene = scene;}

    void scrollUp();
    void scrollDown();

    void toggleShell();
    void out(std::string);
};

#endif // SPARROWSHELL_H
